@@ -32,8 +32,8 @@ from os_brick import utils
 
 LOG = logging.getLogger(__name__)
 
-MULTIPATH_ERROR_REGEX = re.compile("\w{3} \d+ \d\d:\d\d:\d\d \|.*$")
-MULTIPATH_WWID_REGEX = re.compile("\((?P<wwid>.+)\)")
+MULTIPATH_ERROR_REGEX = re.compile(r"\w{3} \d+ \d\d:\d\d:\d\d \|.*$")
+MULTIPATH_WWID_REGEX = re.compile(r"\((?P<wwid>.+)\)")
 MULTIPATH_DEVICE_ACTIONS = ['unchanged:', 'reject:', 'reload:',
                             'switchpg:', 'rename:', 'create:',
                             'resize:']
@@ -101,26 +101,46 @@ class LinuxSCSI(executor.Executor):
         raise exception.VolumePathNotRemoved(volume_path=exist)
 
     def get_device_info(self, device):
-        (out, _err) = self._execute('sg_scan', device, run_as_root=True,
-                                    root_helper=self._root_helper)
         dev_info = {'device': device, 'host': None,
                     'channel': None, 'id': None, 'lun': None}
+        # The input argument 'device' can be of 2 types:
+        # (a) /dev/disk/by-path/XXX which is a symlink to /dev/sdX device
+        # (b) /dev/sdX
+        # If it's a symlink, get the /dev/sdX name first
+        if os.path.islink(device):
+            device = '/dev/' + os.readlink(device).split('/')[-1]
+        # Else it's already a /dev/sdX device.
+        # Then get it from lsscsi output
+        (out, _err) = self._execute('lsscsi')
         if out:
-            line = out.strip()
-            line = line.replace(device + ": ", "")
-            info = line.split(" ")
-
-            for item in info:
-                if '=' in item:
-                    pair = item.split('=')
-                    dev_info[pair[0]] = pair[1]
-                elif 'scsi' in item:
-                    dev_info['host'] = item.replace('scsi', '')
+            for line in out.strip().split('\n'):
+                # The last column of lsscsi is device name
+                if line.split()[-1] == device:
+                    # The first column of lsscsi is [H:C:T:L]
+                    hctl_info = line.split()[0].strip('[]').split(':')
+                    dev_info['host'] = hctl_info[0]
+                    dev_info['channel'] = hctl_info[1]
+                    dev_info['id'] = hctl_info[2]
+                    dev_info['lun'] = hctl_info[3]
+                    break
 
+        LOG.debug('dev_info=%s', str(dev_info))
         return dev_info
 
-    def get_sysfs_wwn(self, device_names):
+    def get_sysfs_wwn(self, device_names, mpath=None):
         """Return the wwid from sysfs in any of devices in udev format."""
+        # If we have a multipath DM we know that it has found the WWN
+        if mpath:
+            # We have the WWN in /uuid even with friendly names, unline /name
+            try:
+                with open('/sys/block/%s/dm/uuid' % mpath) as f:
+                    # Contents are matph-WWN, so get the part we want
+                    wwid = f.read().strip()[6:]
+                    if wwid:  # Check should not be needed, but just in case
+                        return wwid
+            except Exception as exc:
+                LOG.warning('Failed to read the DM uuid: %s', exc)
+
         wwid = self.get_sysfs_wwid(device_names)
         glob_str = '/dev/disk/by-id/scsi-'
         wwn_paths = glob.glob(glob_str + '*')
@@ -128,16 +148,34 @@ class LinuxSCSI(executor.Executor):
         if wwid and glob_str + wwid in wwn_paths:
             return wwid
 
-        # If we have multiple designators follow the symlinks
+        # If we have multiple designators use symlinks to find out the wwn
+        device_names = set(device_names)
         for wwn_path in wwn_paths:
             try:
                 if os.path.islink(wwn_path) and os.stat(wwn_path):
                     path = os.path.realpath(wwn_path)
-                    if path.startswith('/dev/') and path[5:] in device_names:
-                        return wwn_path[len(glob_str):]
+                    if path.startswith('/dev/'):
+                        name = path[5:]
+                        # Symlink may point to the multipath dm if the attach
+                        # was too fast or we took long to check it. Check
+                        # devices belonging to the multipath DM.
+                        if name.startswith('dm-'):
+                            # Get the devices that belong to the DM
+                            slaves_path = '/sys/class/block/%s/slaves' % name
+                            dm_devs = os.listdir(slaves_path)
+                            # This is the right wwn_path if the devices we have
+                            # attached belong to the dm we followed
+                            if device_names.intersection(dm_devs):
+                                break
+
+                        # This is the right wwn_path if  devices we have
+                        elif name in device_names:
+                            break
             except OSError:
                 continue
-        return ''
+        else:
+            return ''
+        return wwn_path[len(glob_str):]
 
     def get_sysfs_wwid(self, device_names):
         """Return the wwid from sysfs in any of devices in udev format."""
@@ -445,8 +483,8 @@ class LinuxSCSI(executor.Executor):
             lines = out.strip()
             lines = lines.split("\n")
             lines = [line for line in lines
-                     if not re.match(MULTIPATH_ERROR_REGEX, line)
-                     and len(line)]
+                     if not re.match(MULTIPATH_ERROR_REGEX, line) and
+                     len(line)]
             if lines:
 
                 mdev_name = lines[0].split(" ")[0]
@@ -531,7 +569,7 @@ class LinuxSCSI(executor.Executor):
                                     root_helper=self._root_helper)
         return out
 
-    def extend_volume(self, volume_paths):
+    def extend_volume(self, volume_paths, use_multipath=False):
         """Signal the SCSI subsystem to test for volume resize.
 
         This function tries to signal the local system's kernel
@@ -561,24 +599,26 @@ class LinuxSCSI(executor.Executor):
             LOG.debug("volume size after scsi device rescan %s", new_size)
 
         scsi_wwn = self.get_scsi_wwn(volume_paths[0])
-        mpath_device = self.find_multipath_device_path(scsi_wwn)
-        if mpath_device:
-            # Force a reconfigure so that resize works
-            self.multipath_reconfigure()
+        if use_multipath:
+            mpath_device = self.find_multipath_device_path(scsi_wwn)
+            if mpath_device:
+                # Force a reconfigure so that resize works
+                self.multipath_reconfigure()
 
-            size = self.get_device_size(mpath_device)
-            LOG.info("mpath(%(device)s) current size %(size)s",
-                     {'device': mpath_device, 'size': size})
-            result = self.multipath_resize_map(scsi_wwn)
-            if 'fail' in result:
-                LOG.error("Multipathd failed to update the size mapping of "
-                          "multipath device %(scsi_wwn)s volume %(volume)s",
-                          {'scsi_wwn': scsi_wwn, 'volume': volume_paths})
-                return None
+                size = self.get_device_size(mpath_device)
+                LOG.info("mpath(%(device)s) current size %(size)s",
+                         {'device': mpath_device, 'size': size})
+                result = self.multipath_resize_map(scsi_wwn)
+                if 'fail' in result:
+                    LOG.error("Multipathd failed to update the size mapping "
+                              "of multipath device %(scsi_wwn)s volume "
+                              "%(volume)s",
+                              {'scsi_wwn': scsi_wwn, 'volume': volume_paths})
+                    return None
 
-            new_size = self.get_device_size(mpath_device)
-            LOG.info("mpath(%(device)s) new size %(size)s",
-                     {'device': mpath_device, 'size': new_size})
+                new_size = self.get_device_size(mpath_device)
+                LOG.info("mpath(%(device)s) new size %(size)s",
+                         {'device': mpath_device, 'size': new_size})
 
         return new_size
 
@@ -626,7 +666,7 @@ class LinuxSCSI(executor.Executor):
         """Find the device name given a session and the hctl.
 
         :param session: A string with the session number
-        "param hctl: An iterable with the host, channel, target, and lun as
+        :param hctl: An iterable with the host, channel, target, and lun as
                      passed to scan.  ie: ('5', '-', '-', '0')
         """
         if '-' in hctl:
