@@ -81,23 +81,26 @@ class RemoteFsClient(executor.Executor):
                             self._get_hash_str(device_name))
 
     def _read_mounts(self):
-        (out, _err) = self._execute('mount', check_exit_code=0)
-        lines = out.split('\n')
-        mounts = {}
-        for line in lines:
-            tokens = line.split()
-            if 2 < len(tokens):
-                device = tokens[0]
-                mnt_point = tokens[2]
-                mounts[mnt_point] = device
-        return mounts
+        """Returns a dict of mounts and their mountpoint
+
+        Format reference:
+        http://man7.org/linux/man-pages/man5/fstab.5.html
+        """
+        with open("/proc/mounts", "r") as mounts:
+            # Remove empty lines and split lines by whitespace
+            lines = [line.split() for line in mounts.read().splitlines()
+                     if line.strip()]
+
+            # Return {mountpoint: mountdevice}.  Fields 2nd and 1st as per
+            # http://man7.org/linux/man-pages/man5/fstab.5.html
+            return {line[1]: line[0] for line in lines if line[0] != '#'}
 
     def mount(self, share, flags=None):
         """Mount given share."""
         mount_path = self.get_mount_point(share)
 
         if mount_path in self._read_mounts():
-            LOG.info('Already mounted: %s', mount_path)
+            LOG.debug('Already mounted: %s', mount_path)
             return
 
         self._execute('mkdir', '-p', mount_path, check_exit_code=0)
@@ -122,7 +125,7 @@ class RemoteFsClient(executor.Executor):
                           run_as_root=True, check_exit_code=0)
         except processutils.ProcessExecutionError as exc:
             if 'already mounted' in exc.stderr:
-                LOG.info("Already mounted: %s", share)
+                LOG.debug("Already mounted: %s", share)
 
                 # The error message can say "busy or already mounted" when the
                 # share didn't actually mount, so look for it.
@@ -158,7 +161,7 @@ class RemoteFsClient(executor.Executor):
     def _check_nfs_options(self):
         """Checks and prepares nfs mount type options."""
         self._nfs_mount_type_opts = {'nfs': self._mount_options}
-        nfs_vers_opt_patterns = ['^nfsvers', '^vers', '^v[\d]']
+        nfs_vers_opt_patterns = ['^nfsvers', '^vers', r'^v[\d]']
         for opt in nfs_vers_opt_patterns:
             if self._option_exists(self._mount_options, opt):
                 return
@@ -218,7 +221,7 @@ class ScalityRemoteFsClient(RemoteFsClient):
         same method signature for class inheritance purpose.
         """
         if self._mount_base in self._read_mounts():
-            LOG.info('Already mounted: %s', self._mount_base)
+            LOG.debug('Already mounted: %s', self._mount_base)
             return
         self._execute('mkdir', '-p', self._mount_base, check_exit_code=0)
         super(ScalityRemoteFsClient, self)._do_mount(
@@ -246,7 +249,7 @@ class VZStorageRemoteFSClient(RemoteFsClient):
 
     def _do_mount(self, mount_type, vz_share, mount_path,
                   mount_options=None, flags=None):
-        m = re.search("(?:(\S+):\/)?([a-zA-Z0-9_-]+)(?::(\S+))?", vz_share)
+        m = re.search(r"(?:(\S+):\/)?([a-zA-Z0-9_-]+)(?::(\S+))?", vz_share)
         if not m:
             msg = (_("Invalid Virtuozzo Storage share specification: %r."
                      "Must be: [MDS1[,MDS2],...:/]<CLUSTER NAME>[:PASSWORD].")
