@@ -12,8 +12,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 import collections
-import mock
 import os
+from unittest import mock
 
 import ddt
 from oslo_concurrency import processutils as putils
@@ -21,6 +21,7 @@ from oslo_concurrency import processutils as putils
 from os_brick import exception
 from os_brick.initiator.connectors import iscsi
 from os_brick.initiator import linuxscsi
+from os_brick.initiator import utils
 from os_brick.privileged import rootwrap as priv_rootwrap
 from os_brick.tests.initiator import test_connector
 
@@ -70,12 +71,17 @@ class ISCSIConnectorTestCase(test_connector.ConnectorTestCase):
                     ('tcp:', 'session3', 'ip3:port3', '1', 'tgt3')]
         self.assertListEqual(expected, res)
 
-    @mock.patch.object(iscsi.ISCSIConnector, '_run_iscsi_session',
-                       return_value=(None, 'error'))
-    def test_get_iscsi_sessions_full_error(self, sessions_mock):
+    @mock.patch.object(iscsi.ISCSIConnector, '_run_iscsi_session')
+    def test_get_iscsi_sessions_full_stderr(self, sessions_mock):
+        iscsiadm_result = ('tcp: [session1] ip1:port1,1 tgt1 (non-flash)\n'
+                           'tcp: [session2] ip2:port2,-1 tgt2 (non-flash)\n'
+                           'tcp: [session3] ip3:port3,1 tgt3\n')
+        sessions_mock.return_value = (iscsiadm_result, 'error')
         res = self.connector._get_iscsi_sessions_full()
-        self.assertEqual([], res)
-        sessions_mock.assert_called()
+        expected = [('tcp:', 'session1', 'ip1:port1', '1', 'tgt1'),
+                    ('tcp:', 'session2', 'ip2:port2', '-1', 'tgt2'),
+                    ('tcp:', 'session3', 'ip3:port3', '1', 'tgt3')]
+        self.assertListEqual(expected, res)
 
     @mock.patch.object(iscsi.ISCSIConnector, '_get_iscsi_sessions_full')
     def test_get_iscsi_sessions(self, sessions_mock):
@@ -913,7 +919,7 @@ Setting up iSCSI targets: unused
         mock_isdir.return_value = False
         expected = []
         actual = self.connector.get_all_available_volumes()
-        self.assertItemsEqual(expected, actual)
+        self.assertCountEqual(expected, actual)
 
     @mock.patch.object(iscsi.ISCSIConnector, '_get_device_path')
     def test_get_potential_paths_mpath(self, get_path_mock):
@@ -982,6 +988,7 @@ Setting up iSCSI targets: unused
             [('tcp:', 'session1', 'ip1:port1', '1', 'tgt'),
              ('tcp:', session, 'ip1:port1', '-1', 'tgt1')]
         ]
+        utils.ISCSI_SUPPORTS_MANUAL_SCAN = None
         with mock.patch.object(self.connector, '_execute') as exec_mock:
             exec_mock.side_effect = [('', 'error'), ('', None),
                                      ('', None), ('', None),
@@ -991,6 +998,7 @@ Setting up iSCSI targets: unused
         # True refers to "manual scans", since the call to update
         # node.session.scan didn't fail they are set to manual
         self.assertEqual((session, True), res)
+        self.assertTrue(utils.ISCSI_SUPPORTS_MANUAL_SCAN)
         prefix = 'iscsiadm -m node -T tgt1 -p ip1:port1'
         expected_cmds = [
             prefix,
@@ -1013,6 +1021,7 @@ Setting up iSCSI targets: unused
             [('tcp:', 'session1', 'Ip1:port1', '1', 'tgt'),
              ('tcp:', session, 'IP1:port1', '-1', 'tgt1')]
         ]
+        utils.ISCSI_SUPPORTS_MANUAL_SCAN = None
         with mock.patch.object(self.connector, '_execute') as exec_mock:
             exec_mock.side_effect = [('', 'error'), ('', None),
                                      ('', None), ('', None),
@@ -1022,6 +1031,7 @@ Setting up iSCSI targets: unused
         # True refers to "manual scans", since the call to update
         # node.session.scan didn't fail they are set to manual
         self.assertEqual((session, True), res)
+        self.assertTrue(utils.ISCSI_SUPPORTS_MANUAL_SCAN)
         prefix = 'iscsiadm -m node -T tgt1 -p ip1:port1'
         expected_cmds = [
             prefix,
@@ -1043,9 +1053,11 @@ Setting up iSCSI targets: unused
         con_props = self.CON_PROPS.copy()
         con_props.update(auth_method='CHAP', auth_username='user',
                          auth_password='pwd')
+        utils.ISCSI_SUPPORTS_MANUAL_SCAN = None
         res = self.connector._connect_to_iscsi_portal(con_props)
         # False refers to "manual scans", so we have manual iscsi scans
         self.assertEqual((session, True), res)
+        self.assertTrue(utils.ISCSI_SUPPORTS_MANUAL_SCAN)
         prefix = 'iscsiadm -m node -T tgt1 -p ip1:port1'
         expected_cmds = [
             prefix,
@@ -1077,7 +1089,7 @@ Setting up iSCSI targets: unused
                        side_effect=(None, 'tgt2'))
     @mock.patch.object(iscsi.ISCSIConnector, '_connect_vol')
     @mock.patch.object(iscsi.ISCSIConnector, '_cleanup_connection')
-    @mock.patch('time.sleep')
+    @mock.patch('os_brick.utils._time_sleep')
     def test_connect_single_volume(self, sleep_mock, cleanup_mock,
                                    connect_mock, get_wwn_mock):
         def my_connect(rescans, props, data):
@@ -1099,6 +1111,27 @@ Setting up iSCSI targets: unused
             (('ip1:port1', 'tgt1', 4),),
             force=True, ignore_errors=True)
 
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'get_sysfs_wwn', return_value='')
+    @mock.patch.object(iscsi.ISCSIConnector, '_connect_vol')
+    @mock.patch.object(iscsi.ISCSIConnector, '_cleanup_connection')
+    @mock.patch('os_brick.utils._time_sleep')
+    def test_connect_single_volume_no_wwn(self, sleep_mock, cleanup_mock,
+                                          connect_mock, get_wwn_mock):
+        def my_connect(rescans, props, data):
+            data['found_devices'].append('sdz')
+
+        connect_mock.side_effect = my_connect
+
+        res = self.connector._connect_single_volume(self.CON_PROPS)
+
+        expected = {'type': 'block', 'scsi_wwn': '', 'path': '/dev/sdz'}
+        self.assertEqual(expected, res)
+        get_wwn_mock.assert_has_calls([mock.call(['sdz'])] * 10)
+        self.assertEqual(10, get_wwn_mock.call_count)
+        sleep_mock.assert_has_calls([mock.call(1)] * 10)
+        self.assertEqual(10, sleep_mock.call_count)
+        cleanup_mock.assert_not_called()
+
     @staticmethod
     def _get_connect_vol_data():
         return {'stop_connecting': False, 'num_logins': 0, 'failed_logins': 0,
@@ -1109,7 +1142,7 @@ Setting up iSCSI targets: unused
                        side_effect=(None, 'tgt2'))
     @mock.patch.object(iscsi.ISCSIConnector, '_connect_vol')
     @mock.patch.object(iscsi.ISCSIConnector, '_cleanup_connection')
-    @mock.patch('time.sleep')
+    @mock.patch('os_brick.utils._time_sleep')
     def test_connect_single_volume_not_found(self, sleep_mock, cleanup_mock,
                                              connect_mock, get_wwn_mock):
 
@@ -1148,7 +1181,7 @@ Setting up iSCSI targets: unused
     @mock.patch.object(linuxscsi.LinuxSCSI, 'multipath_add_path')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'multipath_add_wwid')
     @mock.patch.object(iscsi.ISCSIConnector, '_connect_vol')
-    @mock.patch('time.sleep')
+    @mock.patch('os_brick.utils._time_sleep')
     def test_connect_multipath_volume_all_succeed(self, sleep_mock,
                                                   connect_mock, add_wwid_mock,
                                                   add_path_mock, get_wwn_mock,
@@ -1173,11 +1206,52 @@ Setting up iSCSI targets: unused
         result = list(get_wwn_mock.call_args[0][0])
         result.sort()
         self.assertEqual(['sda', 'sdb', 'sdc', 'sdd'], result)
+        # Check we pass the mpath
+        self.assertIsNone(get_wwn_mock.call_args[0][1])
         add_wwid_mock.assert_called_once_with('wwn')
         self.assertNotEqual(0, add_path_mock.call_count)
         self.assertGreaterEqual(find_dm_mock.call_count, 2)
         self.assertEqual(4, connect_mock.call_count)
 
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'find_sysfs_multipath_dm',
+                       side_effect=[None, 'dm-0'])
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'get_sysfs_wwn', return_value='')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'multipath_add_path')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'multipath_add_wwid')
+    @mock.patch.object(iscsi.ISCSIConnector, '_connect_vol')
+    @mock.patch('os_brick.utils._time_sleep')
+    def test_connect_multipath_volume_no_wwid(self, sleep_mock, connect_mock,
+                                              add_wwid_mock, add_path_mock,
+                                              get_wwn_mock, find_dm_mock):
+        # Even if we don't have the wwn we'll be able to find the multipath
+        def my_connect(rescans, props, data):
+            devs = {'tgt1': 'sda', 'tgt2': 'sdb', 'tgt3': 'sdc', 'tgt4': 'sdd'}
+            data['stopped_threads'] += 1
+            data['num_logins'] += 1
+            dev = devs[props['target_iqn']]
+            data['found_devices'].append(dev)
+            data['just_added_devices'].append(dev)
+
+        connect_mock.side_effect = my_connect
+
+        res = self.connector._connect_multipath_volume(self.CON_PROPS)
+
+        expected = {'type': 'block', 'scsi_wwn': '', 'multipath_id': '',
+                    'path': '/dev/dm-0'}
+        self.assertEqual(expected, res)
+
+        self.assertEqual(3, get_wwn_mock.call_count)
+        result = list(get_wwn_mock.call_args[0][0])
+        result.sort()
+        self.assertEqual(['sda', 'sdb', 'sdc', 'sdd'], result)
+        # Initially mpath we pass is None, but on last call is the mpath
+        mpath_values = [c[1][1] for c in get_wwn_mock._mock_mock_calls]
+        self.assertEqual([None, None, 'dm-0'], mpath_values)
+        add_wwid_mock.assert_not_called()
+        add_path_mock.assert_not_called()
+        self.assertGreaterEqual(find_dm_mock.call_count, 2)
+        self.assertEqual(4, connect_mock.call_count)
+
     @mock.patch.object(linuxscsi.LinuxSCSI, 'find_sysfs_multipath_dm',
                        side_effect=[None, 'dm-0'])
     @mock.patch.object(linuxscsi.LinuxSCSI, 'get_sysfs_wwn',
@@ -1185,7 +1259,7 @@ Setting up iSCSI targets: unused
     @mock.patch.object(linuxscsi.LinuxSCSI, 'multipath_add_path')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'multipath_add_wwid')
     @mock.patch.object(iscsi.ISCSIConnector, '_connect_vol')
-    @mock.patch('time.sleep')
+    @mock.patch('os_brick.utils._time_sleep')
     def test_connect_multipath_volume_all_fail(self, sleep_mock, connect_mock,
                                                add_wwid_mock, add_path_mock,
                                                get_wwn_mock, find_dm_mock):
@@ -1212,7 +1286,7 @@ Setting up iSCSI targets: unused
     @mock.patch.object(linuxscsi.LinuxSCSI, 'multipath_add_path')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'multipath_add_wwid')
     @mock.patch.object(iscsi.ISCSIConnector, '_connect_vol')
-    @mock.patch('time.sleep')
+    @mock.patch('os_brick.utils._time_sleep')
     def test_connect_multipath_volume_some_fail_mp_found(self, sleep_mock,
                                                          connect_mock,
                                                          add_wwid_mock,
@@ -1254,7 +1328,7 @@ Setting up iSCSI targets: unused
     @mock.patch.object(linuxscsi.LinuxSCSI, 'multipath_add_wwid')
     @mock.patch.object(iscsi.time, 'time', side_effect=(0, 0, 11, 0))
     @mock.patch.object(iscsi.ISCSIConnector, '_connect_vol')
-    @mock.patch('time.sleep')
+    @mock.patch('os_brick.utils._time_sleep')
     def test_connect_multipath_volume_some_fail_mp_not_found(self, sleep_mock,
                                                              connect_mock,
                                                              time_mock,
@@ -1298,7 +1372,7 @@ Setting up iSCSI targets: unused
     @mock.patch.object(linuxscsi.LinuxSCSI, 'multipath_add_wwid')
     @mock.patch.object(iscsi.time, 'time', side_effect=(0, 0, 11, 0))
     @mock.patch.object(iscsi.ISCSIConnector, '_connect_vol')
-    @mock.patch('time.sleep', mock.Mock())
+    @mock.patch('os_brick.utils._time_sleep', mock.Mock())
     def test_connect_multipath_volume_all_loging_not_found(self,
                                                            connect_mock,
                                                            time_mock,
@@ -1322,12 +1396,13 @@ Setting up iSCSI targets: unused
         find_dm_mock.assert_not_called()
         self.assertEqual(12, connect_mock.call_count)
 
-    @mock.patch('time.sleep', mock.Mock())
+    @mock.patch('os_brick.utils._time_sleep')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'scan_iscsi')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'device_name_by_hctl',
                        return_value='sda')
     @mock.patch.object(iscsi.ISCSIConnector, '_connect_to_iscsi_portal')
-    def test_connect_vol(self, connect_mock, dev_name_mock, scan_mock):
+    def test_connect_vol(self, connect_mock, dev_name_mock, scan_mock,
+                         sleep_mock):
         lscsi = self.connector._linuxscsi
         data = self._get_connect_vol_data()
         hctl = [mock.sentinel.host, mock.sentinel.channel,
@@ -1350,8 +1425,72 @@ Setting up iSCSI targets: unused
                                     mock.call(mock.sentinel.session,
                                               self.CON_PROPS['target_lun'])])
 
-        scan_mock.assert_called_once_with(*hctl)
+        scan_mock.assert_not_called()
         dev_name_mock.assert_called_once_with(mock.sentinel.session, hctl)
+        sleep_mock.assert_called_once_with(1)
+
+    @mock.patch('os_brick.utils._time_sleep')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'scan_iscsi')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'device_name_by_hctl',
+                       side_effect=(None, None, None, None, 'sda'))
+    @mock.patch.object(iscsi.ISCSIConnector, '_connect_to_iscsi_portal')
+    def test_connect_vol_rescan(self, connect_mock, dev_name_mock, scan_mock,
+                                sleep_mock):
+        lscsi = self.connector._linuxscsi
+        data = self._get_connect_vol_data()
+        hctl = [mock.sentinel.host, mock.sentinel.channel,
+                mock.sentinel.target, mock.sentinel.lun]
+
+        connect_mock.return_value = (mock.sentinel.session, False)
+
+        with mock.patch.object(lscsi, 'get_hctl',
+                               return_value=hctl) as hctl_mock:
+            self.connector._connect_vol(3, self.CON_PROPS, data)
+
+        expected = self._get_connect_vol_data()
+        expected.update(num_logins=1, stopped_threads=1,
+                        found_devices=['sda'], just_added_devices=['sda'])
+        self.assertDictEqual(expected, data)
+
+        connect_mock.assert_called_once_with(self.CON_PROPS)
+        hctl_mock.assert_called_once_with(mock.sentinel.session,
+                                          self.CON_PROPS['target_lun'])
+
+        scan_mock.assert_called_once_with(*hctl)
+        self.assertEqual(5, dev_name_mock.call_count)
+        self.assertEqual(4, sleep_mock.call_count)
+
+    @mock.patch('os_brick.utils._time_sleep')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'scan_iscsi')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'device_name_by_hctl',
+                       side_effect=(None, None, None, None, 'sda'))
+    @mock.patch.object(iscsi.ISCSIConnector, '_connect_to_iscsi_portal')
+    def test_connect_vol_manual(self, connect_mock, dev_name_mock, scan_mock,
+                                sleep_mock):
+        lscsi = self.connector._linuxscsi
+        data = self._get_connect_vol_data()
+        hctl = [mock.sentinel.host, mock.sentinel.channel,
+                mock.sentinel.target, mock.sentinel.lun]
+
+        # Simulate manual scan
+        connect_mock.return_value = (mock.sentinel.session, True)
+
+        with mock.patch.object(lscsi, 'get_hctl',
+                               return_value=hctl) as hctl_mock:
+            self.connector._connect_vol(3, self.CON_PROPS, data)
+
+        expected = self._get_connect_vol_data()
+        expected.update(num_logins=1, stopped_threads=1,
+                        found_devices=['sda'], just_added_devices=['sda'])
+        self.assertDictEqual(expected, data)
+
+        connect_mock.assert_called_once_with(self.CON_PROPS)
+        hctl_mock.assert_called_once_with(mock.sentinel.session,
+                                          self.CON_PROPS['target_lun'])
+
+        self.assertEqual(2, scan_mock.call_count)
+        self.assertEqual(5, dev_name_mock.call_count)
+        self.assertEqual(4, sleep_mock.call_count)
 
     @mock.patch.object(iscsi.ISCSIConnector, '_connect_to_iscsi_portal',
                        return_value=(None, False))
@@ -1364,7 +1503,7 @@ Setting up iSCSI targets: unused
         expected.update(failed_logins=1, stopped_threads=1)
         self.assertDictEqual(expected, data)
 
-    @mock.patch('time.sleep', mock.Mock())
+    @mock.patch('os_brick.utils._time_sleep', mock.Mock())
     @mock.patch.object(linuxscsi.LinuxSCSI, 'scan_iscsi')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'device_name_by_hctl',
                        return_value=None)
@@ -1395,7 +1534,7 @@ Setting up iSCSI targets: unused
             [mock.call(mock.sentinel.session, hctl),
              mock.call(mock.sentinel.session, hctl)])
 
-    @mock.patch('time.sleep', mock.Mock())
+    @mock.patch('os_brick.utils._time_sleep', mock.Mock())
     @mock.patch.object(linuxscsi.LinuxSCSI, 'scan_iscsi')
     @mock.patch.object(iscsi.ISCSIConnector, '_connect_to_iscsi_portal')
     def test_connect_vol_stop_connecting(self, connect_mock, scan_mock):
@@ -1485,17 +1624,67 @@ Setting up iSCSI targets: unused
             mock.call('/dev/disk/by-id/dm-...'),
             mock.call('/dev/disk/by-id/scsi-wwn')])
 
+    @mock.patch('os_brick.utils._time_sleep')
     @mock.patch('os.path.realpath', return_value='/dev/sdz')
     @mock.patch('os.listdir', return_value=['dm-...', 'scsi-...'])
-    def test__get_device_link_not_found(self, listdir_mock, realpath_mock):
+    def test__get_device_link_not_found(self, listdir_mock, realpath_mock,
+                                        mock_time):
         self.assertRaises(exception.VolumeDeviceNotFound,
                           self.connector._get_device_link,
                           'wwn', '/dev/sda', None)
-        listdir_mock.assert_called_once_with('/dev/disk/by-id/')
-        realpath_mock.assert_has_calls([
-            mock.call('/dev/disk/by-id/scsi-wwn'),
-            mock.call('/dev/disk/by-id/dm-...'),
-            mock.call('/dev/disk/by-id/scsi-...')])
+        listdir_mock.assert_has_calls(3 * [mock.call('/dev/disk/by-id/')])
+        self.assertEqual(3, listdir_mock.call_count)
+        realpath_mock.assert_has_calls(
+            3 * [mock.call('/dev/disk/by-id/scsi-wwn'),
+                 mock.call('/dev/disk/by-id/dm-...'),
+                 mock.call('/dev/disk/by-id/scsi-...')])
+        self.assertEqual(9, realpath_mock.call_count)
+
+    @mock.patch('os_brick.utils._time_sleep')
+    @mock.patch('os.path.realpath')
+    @mock.patch('os.listdir', return_value=['dm-...', 'scsi-...'])
+    def test__get_device_link_symlink_found_after_retry(self, mock_listdir,
+                                                        mock_realpath,
+                                                        mock_time):
+        # Return the expected realpath on the third retry
+        mock_realpath.side_effect = [
+            None, None, None, None, None, None, '/dev/sda']
+
+        # Assert that VolumeDeviceNotFound isn't raised
+        self.connector._get_device_link('wwn', '/dev/sda', None)
+
+        # Assert that listdir and realpath have been called correctly
+        mock_listdir.assert_has_calls(2 * [mock.call('/dev/disk/by-id/')])
+        self.assertEqual(2, mock_listdir.call_count)
+        mock_realpath.assert_has_calls(
+            2 * [mock.call('/dev/disk/by-id/scsi-wwn'),
+                 mock.call('/dev/disk/by-id/dm-...'),
+                 mock.call('/dev/disk/by-id/scsi-...')]
+            + [mock.call('/dev/disk/by-id/scsi-wwn')])
+        self.assertEqual(7, mock_realpath.call_count)
+
+    @mock.patch('os_brick.utils._time_sleep')
+    @mock.patch('os.path.realpath')
+    @mock.patch('os.listdir', return_value=['dm-...', 'scsi-...'])
+    def test__get_device_link_symlink_found_after_retry_by_listdir(
+            self, mock_listdir, mock_realpath, mock_time):
+
+        # Return the expected realpath on the second retry while looping over
+        # the devices returned by listdir
+        mock_realpath.side_effect = [
+            None, None, None, None, None, '/dev/sda']
+
+        # Assert that VolumeDeviceNotFound isn't raised
+        self.connector._get_device_link('wwn', '/dev/sda', None)
+
+        # Assert that listdir and realpath have been called correctly
+        mock_listdir.assert_has_calls(2 * [mock.call('/dev/disk/by-id/')])
+        self.assertEqual(2, mock_listdir.call_count)
+        mock_realpath.assert_has_calls(
+            2 * [mock.call('/dev/disk/by-id/scsi-wwn'),
+                 mock.call('/dev/disk/by-id/dm-...'),
+                 mock.call('/dev/disk/by-id/scsi-...')])
+        self.assertEqual(6, mock_realpath.call_count)
 
     @mock.patch.object(iscsi.ISCSIConnector, '_run_iscsiadm_bare')
     def test_get_node_startup_values(self, run_iscsiadm_bare_mock):
@@ -1567,8 +1756,8 @@ Setting up iSCSI targets: unused
         iqn1 = 'iqn.2010-10.org.openstack:%s' % name1
         iqn2 = 'iqn.2010-10.org.openstack:%s' % name2
         iqn3 = 'iqn.2010-10.org.openstack:%s' % name3
-        connection_properties = self.iscsi_connection(vol, [location], [iqn1])
-        recover_connection = self.iscsi_connection(vol, [location], [iqn2])
+        connection_properties = self.iscsi_connection(vol, [location], iqn1)
+        recover_connection = self.iscsi_connection(vol, [location], iqn2)
 
         node_startup1 = "manual"
         node_startup2 = "automatic"
