@@ -14,15 +14,17 @@
 
 import platform
 import sys
+from unittest import mock
 
-import mock
 from oslo_concurrency import processutils as putils
+from oslo_service import loopingcall
 
 from os_brick import exception
 from os_brick.initiator import connector
 from os_brick.initiator.connectors import base
 from os_brick.initiator.connectors import fake
 from os_brick.initiator.connectors import iscsi
+from os_brick.initiator.connectors import nvmeof
 from os_brick.initiator import linuxfc
 from os_brick.privileged import rootwrap as priv_rootwrap
 from os_brick.tests import base as test_base
@@ -31,8 +33,16 @@ MY_IP = '10.0.0.1'
 FAKE_SCSI_WWN = '1234567890'
 
 
+class ZeroIntervalLoopingCall(loopingcall.FixedIntervalLoopingCall):
+    def start(self, interval, initial_delay=None, stop_on_exception=True):
+        return super(ZeroIntervalLoopingCall, self).start(
+            0, 0, stop_on_exception)
+
+
 class ConnectorUtilsTestCase(test_base.TestCase):
 
+    @mock.patch.object(nvmeof.NVMeOFConnector, '_get_system_uuid',
+                       return_value=None)
     @mock.patch.object(iscsi.ISCSIConnector, 'get_initiator',
                        return_value='fakeinitiator')
     @mock.patch.object(linuxfc.LinuxFibreChannel, 'get_fc_wwpns',
@@ -46,6 +56,7 @@ class ConnectorUtilsTestCase(test_base.TestCase):
                                              multipath_result,
                                              mock_wwnns, mock_wwpns,
                                              mock_initiator,
+                                             mock_sysuuid,
                                              host='fakehost'):
         props_actual = connector.get_connector_properties('sudo',
                                                           MY_IP,
@@ -68,7 +79,7 @@ class ConnectorUtilsTestCase(test_base.TestCase):
 
         mock_list = []
         # Make sure every connector is called
-        for item in connector.connector_list:
+        for item in connector._get_connector_list():
             patched = mock.MagicMock()
             patched.platform = platform.machine()
             patched.os_type = sys.platform
@@ -122,6 +133,8 @@ class ConnectorTestCase(test_base.TestCase):
     def setUp(self):
         super(ConnectorTestCase, self).setUp()
         self.cmds = []
+        self.mock_object(loopingcall, 'FixedIntervalLoopingCall',
+                         ZeroIntervalLoopingCall)
 
     def fake_execute(self, *cmd, **kwargs):
         self.cmds.append(" ".join(cmd))
@@ -213,9 +226,6 @@ class ConnectorTestCase(test_base.TestCase):
                                                    arch='s390x')
         self.assertEqual("FibreChannelConnectorS390X", obj.__class__.__name__)
 
-        obj = connector.InitiatorConnector.factory('aoe', None, arch='x86_64')
-        self.assertEqual("AoEConnector", obj.__class__.__name__)
-
         obj = connector.InitiatorConnector.factory(
             'nfs', None, nfs_mount_point_base='/mnt/test')
         self.assertEqual("RemoteFsConnector", obj.__class__.__name__)
@@ -249,10 +259,6 @@ class ConnectorTestCase(test_base.TestCase):
             arch='x86_64')
         self.assertEqual("RemoteFsConnector", obj.__class__.__name__)
 
-        obj = connector.InitiatorConnector.factory(
-            "disco", None, arch='x86_64')
-        self.assertEqual("DISCOConnector", obj.__class__.__name__)
-
         self.assertRaises(exception.InvalidConnectorProtocol,
                           connector.InitiatorConnector.factory,
                           "bogus", None)
