@@ -10,13 +10,53 @@
 # License for the specific language governing permissions and limitations under
 # the License.
 
-import mock
+from unittest import mock
 
+from os_brick import exception
 from os_brick.initiator import linuxrbd
 from os_brick.tests import base
 from os_brick import utils
 
 
+class MockRados(object):
+
+    class Error(Exception):
+        pass
+
+    class ioctx(object):
+        def __init__(self, *args, **kwargs):
+            pass
+
+        def __enter__(self, *args, **kwargs):
+            return self
+
+        def __exit__(self, *args, **kwargs):
+            return False
+
+        def close(self, *args, **kwargs):
+            pass
+
+    class Rados(object):
+
+        def __init__(self, *args, **kwargs):
+            pass
+
+        def __enter__(self, *args, **kwargs):
+            return self
+
+        def __exit__(self, *args, **kwargs):
+            return False
+
+        def connect(self, *args, **kwargs):
+            pass
+
+        def open_ioctx(self, *args, **kwargs):
+            return MockRados.ioctx()
+
+        def shutdown(self, *args, **kwargs):
+            pass
+
+
 class RBDClientTestCase(base.TestCase):
 
     def setUp(self):
@@ -46,6 +86,17 @@ class RBDClientTestCase(base.TestCase):
 
         self.assertEqual(1, mock_rados.Rados.return_value.shutdown.call_count)
 
+    @mock.patch.object(MockRados.Rados, 'connect', side_effect=MockRados.Error)
+    def test_with_client_error(self, _):
+        linuxrbd.rados = MockRados
+        linuxrbd.rados.Error = MockRados.Error
+
+        def test():
+            with linuxrbd.RBDClient('test_user', 'test_pool'):
+                pass
+
+        self.assertRaises(exception.BrickException, test)
+
 
 class RBDVolumeIOWrapperTestCase(base.TestCase):
 
@@ -146,5 +197,20 @@ class RBDVolumeIOWrapperTestCase(base.TestCase):
     def test_fileno(self):
         self.assertRaises(IOError, self.mock_volume_wrapper.fileno)
 
-    def test_close(self):
-        self.mock_volume_wrapper.close()
+    @mock.patch('os_brick.initiator.linuxrbd.rbd')
+    @mock.patch('os_brick.initiator.linuxrbd.rados')
+    @mock.patch.object(linuxrbd.RBDClient, 'disconnect')
+    def test_close(self, rbd_disconnect, mock_rados, mock_rbd):
+        rbd_client = linuxrbd.RBDClient('user', 'pool')
+        rbd_volume = linuxrbd.RBDVolume(rbd_client, 'volume')
+        rbd_handle = linuxrbd.RBDVolumeIOWrapper(
+            linuxrbd.RBDImageMetadata(rbd_volume, 'pool', 'user', None))
+        rbd_handle.close()
+        self.assertEqual(1, rbd_disconnect.call_count)
+
+
+class RBDVolumeTestCase(base.TestCase):
+    def test_name_attribute(self):
+        mock_client = mock.Mock()
+        rbd_volume = linuxrbd.RBDVolume(mock_client, 'volume')
+        self.assertEqual('volume', rbd_volume.name)
